# -*- coding: utf-8 -*-
# ----------------------------------------------
# purpose : 清洗接口
# author : 
# create_time : 2020/7/1 15:39
# update_time : 2020/7/1 15:39
# copyright : Lavector
# ----------------------------------------------
from lavda.backexcutor.analysis_interface import *


class CleanInterface(object):
    pass


class CleanFlag:
    """
    清洗状态标志
    """
    YES = 1
    NO = 0
